// server/index.js
const express = require('express');
const cors = require('cors');
const { PrismaClient } = require('@prisma/client');
const { hashPassword, verifyPassword } = require('./utils/authHelpers'); 
const jwt = require('jsonwebtoken');
// Uncomment the line below if you're using the authenticateToken middleware
// const { authenticateToken } = require('./middleware/authenticateToken');

const app = express();
const prisma = new PrismaClient();

app.use(cors());
app.use(express.json());

// Use an environment variable for the JWT_SECRET
const JWT_SECRET = process.env.JWT_SECRET;

if (!JWT_SECRET) {
    console.error('JWT_SECRET is not set');
    process.exit(1);
}

// Registration endpoint for a student
app.post('/register/student', async (req, res) => {
    try {
        const { username, password } = req.body;
        // Check if username is already taken
        const existingStudent = await prisma.student.findUnique({
            where: { username: username }
        });
        if (existingStudent) {
            return res.status(409).json({ message: 'Username already exists' });
        }
        const hashedPassword = await hashPassword(password);
        const student = await prisma.student.create({
            data: { username, password: hashedPassword }
        });
        // Exclude password from the response for security reasons
        const { password: _, ...studentWithoutPassword } = student;
        res.status(201).json(studentWithoutPassword);
    } catch (error) {
        res.status(500).json({ message: 'Error registering student', error: error.message });
    }
});

// Registration endpoint for a teacher
app.post('/register/teacher', async (req, res) => {
    try {
        const { username, password } = req.body;
        // Check if username is already taken
        const existingTeacher = await prisma.teacher.findUnique({
            where: { username }
        });
        if (existingTeacher) {
            return res.status(409).json({ message: 'Username already exists' });
        }
        const hashedPassword = await hashPassword(password);
        const teacher = await prisma.teacher.create({
            data: { username, password: hashedPassword }
        });
        // Exclude password from the response for security reasons
        const { password: _, ...teacherWithoutPassword } = teacher;
        res.status(201).json(teacherWithoutPassword);
    } catch (error) {
        res.status(500).json({ message: 'Error registering teacher', error: error.message });
    }
});

// Login endpoint for a student
app.post('/login/student', async (req, res) => {
    try {
        const { username, password } = req.body;
        const student = await prisma.student.findUnique({
            where: { username }
        });
        if (!student || !(await verifyPassword(password, student.password))) {
            return res.status(401).json({ message: 'Invalid credentials' });
        }
        const token = jwt.sign({ studentId: student.studentID }, 'YOUR_SECRET_KEY', { expiresIn: '1h' });
        res.json({ token });
    } catch (error) {
        res.status(500).json({ message: 'Error logging in', error: error.message });
    }
});

// Similar login endpoint for teachers
app.post('/login/teacher', async (req, res) => {
    const { username, password } = req.body;
    try {
        const teacher = await prisma.teacher.findUnique({ where: { username } });
        if (!teacher) {
            return res.status(401).json({ message: 'Invalid username or password' });
        }
    
        const isPasswordValid = await verifyPassword(password, teacher.password);
        if (!isPasswordValid) {
            return res.status(401).json({ message: 'Invalid username or password' });
        }
    
        const token = jwt.sign({ teacherId: teacher.teacherID }, process.env.JWT_SECRET, { expiresIn: '1h' });
        res.json({ token });
        } catch (err) {
        res.status(500).json({ message: 'Something went wrong', error: err.message });
        }
    });

    /* Commented out due to repeated Logic Conflicts
// Example of a protected route
app.get('/protected-route', authenticateToken, (req, res) => {
    res.json({ message: 'This is a protected route' });
});
*/

// Start the server
const PORT = process.env.PORT || 5000;
app.listen(PORT, () => {
    console.log(`Server running on port ${PORT}`);
});
